"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs2/helpers/interopRequireDefault");

var _Object$defineProperty = require("@babel/runtime-corejs2/core-js/object/define-property");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.hasNewLauncherVersion = hasNewLauncherVersion;
exports.fetchChannels = fetchChannels;
exports.fetchReleases = fetchReleases;

var _regenerator = _interopRequireDefault(require("@babel/runtime-corejs2/regenerator"));

var _stringify = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/json/stringify"));

var _promise = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/promise"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime-corejs2/helpers/asyncToGenerator"));

var _config = _interopRequireDefault(require("./config"));

var _tq_firebase = require("./tq_firebase");

var app = require('electron').remote.app;

var log = require('electron-log');

var jetpack = require('fs-jetpack');

var path = require('path'); // Check if there is a newer version of the launcher application available


function hasNewLauncherVersion() {
  return _hasNewLauncherVersion.apply(this, arguments);
}

function _hasNewLauncherVersion() {
  _hasNewLauncherVersion = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee3() {
    var version, doc, latest;
    return _regenerator["default"].wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            version = app.getVersion().split('.')[0];
            _context3.next = 3;
            return _tq_firebase.db.collection('launcherVersion').doc('latest').get();

          case 3:
            doc = _context3.sent;
            latest = doc.data().version;
            console.log('version:', version);
            console.log('latest:', latest);
            return _context3.abrupt("return", version !== latest);

          case 8:
          case "end":
            return _context3.stop();
        }
      }
    }, _callee3);
  }));
  return _hasNewLauncherVersion.apply(this, arguments);
}

function fetchChannels() {
  function doFetch(_x, _x2) {
    return _doFetch.apply(this, arguments);
  }

  function _doFetch() {
    _doFetch = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee(resolve, reject) {
      var cachedChannelsList, cacheFile, channelsJson, channelList, snapshot;
      return _regenerator["default"].wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              cachedChannelsList = null;
              _context.prev = 1;
              // Load cached list of channels, if available.
              cacheFile = path.resolve(_config["default"].cacheDir, "channels.json");

              if (jetpack.exists(cacheFile)) {
                channelsJson = jetpack.read(cacheFile);

                if (channelsJson) {
                  cachedChannelsList = JSON.parse(channelsJson);
                }
              } // Grab latest channel list from Firebase


              channelList = [];
              _context.next = 7;
              return _tq_firebase.db.collection('releaseChannels').orderBy('priority').get();

            case 7:
              snapshot = _context.sent;
              snapshot.forEach(function (doc) {
                var docData = doc.data();
                docData.id = doc.id;
                channelList.push(docData);
              }); // Store the most recent result in a file to use as a fallback
              // when offline, IF we successfully actually got something from
              // Firebase

              if (channelList && channelList.length > 0) {
                jetpack.write(cacheFile, (0, _stringify["default"])(channelList));
                resolve(channelList);
              } else if (cachedChannelsList) {
                resolve(cachedChannelsList);
              } else {
                resolve([]);
              }

              _context.next = 18;
              break;

            case 12:
              _context.prev = 12;
              _context.t0 = _context["catch"](1);
              log.error("Error fetching channel list", _context.t0);

              if (!cachedChannelsList) {
                _context.next = 17;
                break;
              }

              return _context.abrupt("return", resolve(cachedChannelsList));

            case 17:
              reject(_context.t0);

            case 18:
            case "end":
              return _context.stop();
          }
        }
      }, _callee, null, [[1, 12]]);
    }));
    return _doFetch.apply(this, arguments);
  }

  return new _promise["default"](function (resolve, reject) {
    doFetch(resolve, reject);
  });
}

function fetchReleases(channelName) {
  function doFetch(_x3, _x4) {
    return _doFetch2.apply(this, arguments);
  }

  function _doFetch2() {
    _doFetch2 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee2(resolve, reject) {
      var cachedReleaseList, cacheFile, releaseJson, releaseList, snapshot;
      return _regenerator["default"].wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              cachedReleaseList = null;
              _context2.prev = 1;
              cacheFile = path.resolve(_config["default"].cacheDir, "releases-".concat(channelName, ".json"));

              if (jetpack.exists(cacheFile)) {
                releaseJson = jetpack.read(cacheFile);

                if (releaseJson) {
                  cachedReleaseList = JSON.parse(releaseJson);
                }
              } // Grab the last several releases 


              releaseList = [];
              _context2.next = 7;
              return _tq_firebase.db.collection('releases').where('channelName', '==', channelName).orderBy('createdAt', 'desc').orderBy('name', 'desc').get();

            case 7:
              snapshot = _context2.sent;
              snapshot.forEach(function (doc) {
                // Massage release data into the format we want
                var docData = {
                  id: doc.id,
                  createdAt: doc.get('createdAt').toDate(),
                  name: doc.get('name'),
                  checksums: doc.get('checksums'),
                  channelName: channelName
                };
                releaseList.push(docData);
              }); // Store the most recent result in a file to use as a fallback
              // when offline, IF we successfully actually got something from
              // Firebase

              if (releaseList && releaseList.length > 0) {
                jetpack.write(cacheFile, (0, _stringify["default"])(releaseList));
                resolve(releaseList);
              } else if (cachedReleaseList) {
                resolve(cachedReleaseList);
              } else {
                resolve([]);
              }

              _context2.next = 18;
              break;

            case 12:
              _context2.prev = 12;
              _context2.t0 = _context2["catch"](1);
              log.error("Error fetching release list", _context2.t0);

              if (!cachedReleaseList) {
                _context2.next = 17;
                break;
              }

              return _context2.abrupt("return", resolve(cachedReleaseList));

            case 17:
              reject(_context2.t0);

            case 18:
            case "end":
              return _context2.stop();
          }
        }
      }, _callee2, null, [[1, 12]]);
    }));
    return _doFetch2.apply(this, arguments);
  }

  return new _promise["default"](function (resolve, reject) {
    doFetch(resolve, reject);
  });
}
//# sourceMappingURL=version_manager.js.map