import { remote } from 'electron';
import isDev from 'electron-is-dev';
import * as jetpack from 'fs-jetpack';
import * as path from 'path';

const log = require('electron-log');

function getWritablePath() {
  let writablePath = remote.app.getAppPath();

  // Confirm we have access to write to this directory
  try {
    const testPath = path.resolve(writablePath, 'test');
    jetpack.remove(testPath);
    jetpack.dir(testPath);
    jetpack.write(path.resolve(testPath, '.test'), 'success');
    jetpack.remove(testPath);
  } catch (e) {
    // If we can't, then we'll use the user directory
    writablePath = remote.app.getPath('appData');
  }

  return writablePath;
}

// Default configuration
let config = {
  appBaseUrl: 'https://twilioquest-prod.firebaseapp.com/quest/next',
  launcherUrl: '/launcher',
  cacheDir: path.resolve(getWritablePath(), '_cache'),

  // Firebase config for production - since this is the non-admin SDK intended
  // for use in browser JS applications, it is safe to store configuration in
  // version control.
  firebaseConfig: {
    apiKey: 'AIzaSyDI5g9d6XTQtUWT6tPqFNJf171sQ08N-ZA',
    authDomain: 'twilioquest-prod.firebaseapp.com',
    databaseURL: 'https://twilioquest-prod.firebaseio.com',
    projectId: 'twilioquest-prod',
    storageBucket: 'twilioquest-prod.appspot.com',
    messagingSenderId: '397868221836'
  }
};

// Attempt to load local config in the dev environment, if present
if (isDev) {
  try {
    const localConfig = require('./local_config').default;
    config = Object.assign(config, localConfig);
  } catch (e) {
    log.warn('Error loading local config:');
    log.warn(e);
  }
}

jetpack.dir(config.cacheDir);

export default config;
